/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "classhierarchy.h"
#include "classless.h"
#include "mainprogelems.h"
#include "mainglobalvars.h"
#include "mainverbose.h"
#include "mainerrormanager.h"

#include <iostream>
using std::cout;
using std::endl;

#include <queue>
#include <vector>

using std::priority_queue;

ProgElems *prog_elems;
Verbose *verbose;
ErrorManager *errormanager;

int line;
int yylloc;

typedef priority_queue<const ClassDecl *, std::vector<const ClassDecl *>, Class_less> MyHierarchy;
MyHierarchy class_hierarchy;

int
main()
{
  errormanager = new ErrorManager;

  /*
    The class hierarchy is as follows

    T1

    T2    T3

    T2b   T4

          T5

    T6 inherits from both T2 and T5
   */

  ClassDecl *T1 = new ClassDecl ("T1", 0);

  BaseClasses *T2Bases = new BaseClasses;
  BaseClass *BaseT1 = new BaseClass ("T1", true, NONE);
  BaseT1->setClassDecl(T1);
  T2Bases->push_back (BaseT1);
  ClassDecl *T2 = new ClassDecl ("T2", T2Bases);
  ClassDecl *T3 = new ClassDecl ("T3", T2Bases);

  BaseClasses *T4Bases = new BaseClasses;
  BaseClass *BaseT3 = new BaseClass ("T3", true, NONE);
  BaseT3->setClassDecl (T3);
  T4Bases->push_back (BaseT3);
  ClassDecl *T4 = new ClassDecl ("T4", T4Bases);

  BaseClasses *T5Bases = new BaseClasses;
  BaseClass *BaseT4 = new BaseClass ("T4", true, NONE);
  BaseT4->setClassDecl (T4);
  T5Bases->push_back (BaseT4);
  ClassDecl *T5 = new ClassDecl ("T5", T5Bases);

  BaseClasses *T2bBases = new BaseClasses;
  BaseClass *BaseT2 = new BaseClass ("T2", true, NONE);
  BaseT2->setClassDecl (T2);
  T2bBases->push_back (BaseT2);
  ClassDecl *T2b = new ClassDecl ("T2b", T2bBases);

  BaseClasses *T6Bases = new BaseClasses;
  T6Bases->push_back (BaseT2);
  BaseClass *BaseT5 = new BaseClass ("T5", true, NONE);
  BaseT5->setClassDecl (T5);
  T6Bases->push_back (BaseT5);
  ClassDecl *T6 = new ClassDecl ("T6", T6Bases);

  cout << "Insert: T2, T3, T4, T5, T6, T2b" << endl;
  ClassHierarchy hierarchy (T1);
  hierarchy.insert(T2);
  hierarchy.insert(T3);
  hierarchy.insert(T4);
  hierarchy.insert(T5);
  hierarchy.insert(T6);
  hierarchy.insert(T2b);

  class_hierarchy.push(T1);
  class_hierarchy.push(T2);
  class_hierarchy.push(T3);
  class_hierarchy.push(T4);
  class_hierarchy.push(T5);
  class_hierarchy.push(T6);
  class_hierarchy.push(T2b);

  for (HierarchyIterator it = hierarchy.begin(); it != hierarchy.end(); ++it)
    cout << "class " << (*it)->getClassName() << endl;

  cout << "PRIORITY QUEUE:" << endl;
  while (! class_hierarchy.empty())
    {
      cout << "class " << class_hierarchy.top()->getClassName() << endl;
      class_hierarchy.pop();
    }

  cout << "Insert: T2b, T3, T4, T5, T6, T2" << endl;
  ClassHierarchy hierarchy2 (T1);
  hierarchy2.insert(T2b);
  hierarchy2.insert(T3);
  hierarchy2.insert(T4);
  hierarchy2.insert(T5);
  hierarchy2.insert(T6);
  hierarchy2.insert(T2);

  class_hierarchy.push(T1);
  class_hierarchy.push(T2b);
  class_hierarchy.push(T3);
  class_hierarchy.push(T4);
  class_hierarchy.push(T5);
  class_hierarchy.push(T6);
  class_hierarchy.push(T2);

  for (HierarchyIterator it = hierarchy2.begin(); it != hierarchy2.end(); ++it)
    cout << "class " << (*it)->getClassName() << endl;

  cout << "PRIORITY QUEUE:" << endl;
  while (! class_hierarchy.empty())
    {
      cout << "class " << class_hierarchy.top()->getClassName() << endl;
      class_hierarchy.pop();
    }

  cout << "Insert: T6, T5, T4, T2b, T3, T3, T2" << endl;
  ClassHierarchy hierarchy3 (T1);
  hierarchy3.insert(T6);
  hierarchy3.insert(T5);
  hierarchy3.insert(T4);
  hierarchy3.insert(T2b);
  hierarchy3.insert(T3);
  hierarchy3.insert(T3);
  hierarchy3.insert(T2);

  class_hierarchy.push(T1);
  class_hierarchy.push(T6);
  class_hierarchy.push(T5);
  class_hierarchy.push(T4);
  class_hierarchy.push(T2b);
  class_hierarchy.push(T3);
  class_hierarchy.push(T3);
  class_hierarchy.push(T2);

  for (HierarchyIterator it = hierarchy3.begin(); it != hierarchy3.end(); ++it)
    cout << "class " << (*it)->getClassName() << endl;

  cout << "PRIORITY QUEUE:" << endl;
  while (! class_hierarchy.empty())
    {
      cout << "class " << class_hierarchy.top()->getClassName() << endl;
      class_hierarchy.pop();
    }

  return 0;
}
