/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/*
  It is basically the same of deriv2_private.h, but in this case both Base1
  and Deriv1 define the multi method as protected, so Deriv2 receives
  only the branches defined in itself
 */

#ifndef DERIV2_PROTECTED_H
#define DERIV2_PROTECTED_H

#include <iostream>

#line 22 "deriv2_protected2.hhh"
class T1
{
 public:
  virtual void foo() {}
  virtual ~T1() {}
};

#line 29 "deriv2_protected2.hhh"
class T1_d : public T1
{
  void foo() {}
};

#line 34 "deriv2_protected2.hhh"
class T2 : public T1_d
{
  void foo2() {}
};

#line 39 "deriv2_protected2.hhh"
class T3 : public T2
{
  void foo3() {}
};

#line 44 "deriv2_protected2.hhh"
class T4 : public T3
{
  void foo4() {}
};

// doublecpp: forward declarations, DO NOT MODIFY
class T1; // file: deriv2_protected2.hhh
class T2; // file: deriv2_protected2.hhh
// doublecpp: end, DO NOT MODIFY

#line 49 "deriv2_protected2.hhh"
class Base1
{
 protected:
// doublecpp: method branches, DO NOT MODIFY
#line 53 "deriv2_protected2.hhh"
virtual void m(T1 * t);
#line 54 "deriv2_protected2.hhh"
virtual void m(T2 * t);
virtual void m_DB(T1 * t);
#line 57 "deriv2_protected2.hhh"
// doublecpp: end, DO NOT MODIFY


// doublecpp: method branches, DO NOT MODIFY
#line 61 "deriv2_protected2.hhh"
virtual void n(T1 * t);
#line 62 "deriv2_protected2.hhh"
virtual void n(T2 * t);
#line 67 "deriv2_protected2.hhh"
virtual void n(T1 * t, int i);
#line 68 "deriv2_protected2.hhh"
virtual void n(T2 * t, int i);
virtual void n_DB(T1 * t);
virtual void n_DB(T1 * t, int i);
#line 71 "deriv2_protected2.hhh"
// doublecpp: end, DO NOT MODIFY


 public:
  virtual void pub_m(T1 *t)
  {
    // in spite of m_DB being protected in Deriv1, dynamic binding
    // will still be applied when pub_m is invoked on an instance of Deriv1
    m_DB (t);
  }

  virtual void pub_n(T1 *t)
  {
    n_DB (t);
  }

  virtual void pub_n(T1 *t, int i)
  {
    n_DB (t,i);
  }

  virtual ~Base1() {}
};

// doublecpp: forward declarations, DO NOT MODIFY
class T1; // file: deriv2_protected2.hhh
class T3; // file: deriv2_protected2.hhh
// doublecpp: end, DO NOT MODIFY

#line 95 "deriv2_protected2.hhh"
class Deriv1 : public Base1
{
 protected:
  // notice that in this class the method is private
// doublecpp: method branches, DO NOT MODIFY
using Base1::m;
#line 100 "deriv2_protected2.hhh"
void m(T1 * t);
#line 101 "deriv2_protected2.hhh"
void m(T3 * t);
void m_DB(T1 * t);
#line 101 "deriv2_protected2.hhh"
// doublecpp: end, DO NOT MODIFY


// doublecpp: method branches, DO NOT MODIFY
using Base1::n;
#line 105 "deriv2_protected2.hhh"
void n(T1 * tt);
#line 106 "deriv2_protected2.hhh"
void n(T3 * t, int i);
void n_DB(T1 * tt);
void n_DB(T1 * t, int i);
#line 106 "deriv2_protected2.hhh"
// doublecpp: end, DO NOT MODIFY

};

// doublecpp: forward declarations, DO NOT MODIFY
class T4; // file: deriv2_protected2.hhh
// doublecpp: end, DO NOT MODIFY

#line 110 "deriv2_protected2.hhh"
class Deriv2 : public Deriv1
{
 public:
// doublecpp: method branches, DO NOT MODIFY
using Deriv1::m;
#line 114 "deriv2_protected2.hhh"
void m(T4 * t);
void m_DB(T1 * t);
#line 114 "deriv2_protected2.hhh"
// doublecpp: end, DO NOT MODIFY


// doublecpp: method branches, DO NOT MODIFY
using Deriv1::n;
#line 118 "deriv2_protected2.hhh"
void n(T4 * t, int i);
void n_DB(T1 * t, int i);
#line 118 "deriv2_protected2.hhh"
// doublecpp: end, DO NOT MODIFY


  void my_pub(T1 *t)
  {
    m_DB(t);
  }
};

#endif
