#include <iostream>
#include <string>
#include <sstream>
#include <list>

#include "gen_test_cmd.h"
#include "target_class.h"
#include "target_object.h"
#include "client_class.h"
#include "branch_def.h"

using namespace std;

typedef list<string> TargetList;

#define TARGET_CLASS_NAME "T"

string target_prefix;
int hierarchy_depth;
int hierarchy_width;
target_class_gen_class target_gen;
TargetList target_list;

void
generate_target_class(const string &base, int depth, int width)
{
  if (depth > hierarchy_depth)
    return;

  if (width > hierarchy_width)
    return;

  ostringstream class_suffix;
  string inherit_statement;

  if (depth > 0)
    {
      class_suffix << base << "_" << width;

      inherit_statement = ": public " + 
        target_prefix + TARGET_CLASS_NAME + base;        
    }

  target_gen.set_inherit (inherit_statement);
  target_gen.set_suffix (class_suffix.str ());
  target_gen.generate_target_class (cout);

  target_list.push_back (class_suffix.str ());

  for (int w = 1; w <= hierarchy_width; ++w)
    generate_target_class (class_suffix.str (), depth + 1, w);
}

int
main(int argc, char *argv[])
{
  gengetopt_args_info args_info;

  if(cmdline_parser(argc, argv, &args_info) != 0)
    // calls cmdline parser. The user gived bag args if it doesn't return -1
    return 1;

  cout << "#ifndef FOO_H" << endl;
  cout << "#define FOO_H" << endl;

  target_prefix = 
    (args_info.target_prefix_given ? args_info.target_prefix_arg : "");
  hierarchy_depth = args_info.depth_arg;
  hierarchy_width = args_info.width_arg;

  target_gen.set_prefix (target_prefix);  
  generate_target_class ("", 0, 0);

  client_class_gen_class client_gen;
  branch_def_gen_class branch_gen;
  target_object_gen_class target_obj_gen;
  ostringstream branches;
  ostringstream invocations;
  ostringstream num_of_invocations;
  ostringstream target_obj_decls;

  client_gen.set_methname ("m");
  branch_gen.set_classname ("Client");
  branch_gen.set_methname ("m");
  target_obj_gen.set_targetclass (target_prefix + TARGET_CLASS_NAME);
  for (TargetList::iterator it = target_list.begin ();
       it != target_list.end (); ++it)
    {
      string t_class = target_prefix + TARGET_CLASS_NAME + (*it);
      branch_gen.set_target (t_class);
      branch_gen.generate_branch_def (branches);
      target_obj_gen.set_targettype (t_class);
      target_obj_gen.generate_target_object (target_obj_decls, 2);
      invocations << "client.m_DB(target_" << t_class << ");\n";
    }

  num_of_invocations << target_list.size ();

  client_gen.set_branches (branches.str ());
  client_gen.set_targetobjects (target_obj_decls.str ());
  client_gen.set_invocations (invocations.str ());
  client_gen.set_methnum (num_of_invocations.str ());
  client_gen.generate_client_class (cout);

  cout << "#endif // FOO_H" << endl;
  return 0;
}
